'''LSST JupyterHub utilities and helpers.

These implement the LSST-specific tooling for the LSST Science
Platform Notebook Aspect.  The repo scanner looks for Docker images in
a repository with a particular tag format; the prepuller pulls a
subset of those images to each node.  The reaper removes images past a
certain age, based on the tag format.  The LSST Manager class provides
a hierarchy of objects that hold LSST-specific configuration and logic
for spawning JupyterLab pods, and the spawner and authenticators
provide the pod spawner and the LSST-supported authentication methods
and logic.  Convenience functions are in 'utils' and JupyterHub
configuration convenience functions are in 'config_helpers'.
'''
from .singleton import Singleton
from .scanrepo import ScanRepo, SingletonScanner, Prepuller, Reaper
from .lsstmgr import LSSTMiddleManager, check_membership
from .spawner import LSSTSpawner
from .authenticator.lsstcilogonauth import LSSTCILogonOAuthenticator
from .authenticator.lsstgithubauth import LSSTGitHubOAuthenticator
from .authenticator.lsstjwtauth import LSSTJWTAuthenticator
from .utils import (rreplace, sanitize_dict, get_execution_namespace,
                    make_logger, str_bool, str_true, listify, intify,
                    floatify, list_duplicates)
from .config import LSSTConfig
from .setup_auth import configure_auth_and_spawner
from ._version import __version__

__all__ = [LSSTMiddleManager, check_membership, Prepuller, Reaper,
           ScanRepo, Singleton, SingletonScanner, LSSTSpawner,
           LSSTCILogonOAuthenticator, LSSTGitHubOAuthenticator,
           LSSTJWTAuthenticator, rreplace, sanitize_dict,
           get_execution_namespace, make_logger, str_bool, str_true,
           listify, intify, floatify, list_duplicates, LSSTConfig,
           configure_auth_and_spawner, __version__]



'''LSST-specific CILogon OAuthenticator class, delegating its LSST-specific
authentication logic to its auth_mgr.
'''
import oauthenticator
from tornado import gen
from .lsstauth import LSSTAuthenticator
from ..lsstmgr import check_membership
from ..utils import make_logger


class LSSTCILogonOAuthenticator(LSSTAuthenticator,
                                oauthenticator.CILogonOAuthenticator):
    login_handler = oauthenticator.CILogonLoginHandler
    _default_domain = None

    def __init__(self, *args, **kwargs):
        self.log = make_logger()
        self.log.debug("Creating LSSTCILogonOAuthenticator")
        super().__init__(*args, **kwargs)

    @gen.coroutine
    def authenticate(self, handler, data=None):
        '''Authenticate and set UID/groups on auth_mgr.
        '''
        self.log.info("Authenticating user against CILogon.")
        cfg = self.lsst_mgr.config
        userdict = yield super().authenticate(handler, data)
        if not userdict:
            return None
        ast = yield self.user.get_auth_state()
        uid, groupmap = self.resolve_cilogon(ast['cilogon_user'])
        groups = groupmap.keys()
        membership = check_membership(groups, cfg.allowed_groups,
                                      cfg.forbidden_groups, log=self.log)
        if not membership:
            self.log.warning("Group membership check failed.")
            return None
        ast['group_map'] = groupmap
        user_rec = ast["cilogon_user"]
        username = user_rec["uid"]
        if "eppn" in user_rec:
            username, domain = user_rec["eppn"].split("@")
        else:
            domain = ""
        if domain and self._default_domain and domain != self._default_domain:
            username = username + "." + domain
            userdict["name"] = username
        return userdict



#!/usr/bin/env python3
from ..config import LSSTConfig
from .parse_args import parse_args
from .prepuller import Prepuller


def prepullerstandalone():
    '''Run prepuller process as a standalone command.
    '''
    lc = LSSTConfig()
    args = parse_args(cfg=lc, desc="Set up DaemonSets to prepull.",
                      component="prepuller")
    prepuller = Prepuller(args=args)
    prepuller.update_images_from_repo()
    prepuller.build_nodelist()
    prepuller.build_pod_specs()
    prepuller.clean_completed_pods()
    prepuller.run_pods()


if __name__ == "__main__":
    prepullerstandalone()
